# Pyrogram - Telegram MTProto API Client Library for Python
# Copyright (C) 2017-2019 Dan Tès <https://github.com/delivrance>
#
# This file is part of Pyrogram.
#
# Pyrogram is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Pyrogram is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

from ..object import Object


class InputMedia(Object):
    """Content of a media message to be sent.

    It should be one of:

    - :obj:`InputMediaAnimation`
    - :obj:`InputMediaDocument`
    - :obj:`InputMediaAudio`
    - :obj:`InputMediaPhoto`
    - :obj:`InputMediaVideo`
    """

    def __init__(self, media: str, file_ref: str, caption: str, parse_mode: str):
        super().__init__()

        self.media = media
        self.file_ref = file_ref
        self.caption = caption
        self.parse_mode = parse_mode
